<?php

/*
	Players are the living, breathing entities that interact with the game world. They are inextricably linked to their
	accounts, and are the primary means by which the player interacts with the game world. Separating the player from
	the account allows for multiple players to be associated with a single account, and to prevent concurrency issues
	when performing auth checks on the database.

	When creating a player, we want to init all of the related data tables; wallets, inventory, bank, etc.

	When retrieving a player, we will get the tables as-needed, to prevent allocating more memory than we need.
*/

/**
 * Create a player. Only a user ID and a name are required. All other fields are optional. Pass a key-value array
 * of overrides to set additional fields. A player's name must be unique, but this function does not check for that.
 */
function player_create(int $user_id, string $name, array $overrides = []): int
{
	// Prep the data and merge in any overrides
	$data = ['user_id' => $user_id, 'name' => $name];
	if (!empty($overrides)) $data = array_merge($data, $overrides);

	// Prep the fields for the query
	$k = array_keys($data);
	$f = implode(', ', array_keys($k));
	$v = implode(', ', array_map(fn($x) => ":$x", $k));

	// Create the player!
	if (db_query(db_live(), "INSERT INTO players ($f) VALUES ($v)", $data) === false) {
		// @TODO: Log this error
		throw new Exception('Failed to create player.');
	}

	// Get the player ID
	return db_live()->lastInsertRowID();
}
